@@ -4,7 +4,7 @@ from qe_tools.parsers import CpInputFile as BaseCpInputFile
 from .base import StructureParseMixin
 
 
-class CpInputFile(StructureParseMixin, BaseCpInputFile):
+class CpInputFile(StructureParseMixin, BaseCpInputFile):  # pylint: disable=too-few-public-methods
     """Parser of Quantum ESPRESSO cp.x input file into AiiDA nodes.
 
     .. note:: This mixes in :class:`~aiida_quantumespresso.tools.base.StructureParseMixin` which adds the functionality
