@@ -2,8 +2,8 @@
 """TCOD export plugin for `CpCalculations`."""
 try:
     from aiida_tcod.tools.dbexporters.tcod import BaseTcodtranslator  # pylint: disable=import-error
-except ImportError:
-    raise ImportError('dependency `aiida-tcod` not installed; run `pip install aiida-tcod` to do so.')
+except ImportError as exception:
+    raise ImportError('dependency `aiida-tcod` not installed; run `pip install aiida-tcod` to do so.') from exception
 
 
 class CpTcodtranslator(BaseTcodtranslator):
